==> localstack_ext-4.1.1.dev1.tar.gz: localstack_ext-4.1.1.dev1/localstack/pro/core/cli/replicator.py <==
_F='--delay'
_E='profile_name'
_D='region_name'
_C='aws_secret_access_key'
_B='aws_access_key_id'
_A=None
import os,time
from configparser import ConfigParser
from pathlib import Path
from typing import Mapping,Optional,TypedDict
import click,requests
from localstack import config
from localstack.cli import console
from localstack.cli.exceptions import CLIError
from..config import ENABLE_REPLICATOR
from.cli import RequiresLicenseGroup,_assert_host_reachable
AWS_CONFIG_ENV_VARS={_B:'{}_ACCESS_KEY_ID',_C:'{}_SECRET_ACCESS_KEY','aws_session_token':'{}_SESSION_TOKEN',_D:'{}_DEFAULT_REGION','endpoint_url':'{}_ENDPOINT_URL',_E:'{}_PROFILE'}
class ProfileLoadError(RuntimeError):
	def __init__(A,profile_name):super().__init__(f"Could not find profile '{profile_name}'")
class AWSConfig(TypedDict,total=False):aws_access_key_id:str;aws_secret_access_key:str;aws_session_token:Optional[str];region_name:str;endpoint_url:Optional[str];profile_name:str
def get_aws_env_config(prefix):A={A:os.getenv(B.format(prefix))for(A,B)in AWS_CONFIG_ENV_VARS.items()};return AWSConfig(**{B:A for(B,A)in A.items()if A})
def get_config_from_profile(profile_name,profile_dir=_A):
	B=profile_name;A=profile_dir;A=A or Path.home()/'.aws'
	def C(path,profile_prefix=''):
		A=ConfigParser();A.read(path)
		try:return A[f"{profile_prefix}{B}"]
		except KeyError:raise ProfileLoadError(B)
	E=C(A/'config',profile_prefix='profile ');D=C(A/'credentials');return AWSConfig(aws_access_key_id=D[_B],aws_secret_access_key=D[_C],region_name=E['region'],profile_name=B)
def get_source_config():
	A=get_aws_env_config('AWS')
	if(B:=A.get(_E)):
		try:A=get_config_from_profile(B)
		except ProfileLoadError as C:raise CLIError(f"{C}. Profile specified with 'AWS_PROFILE' environment variable.")
	else:
		if not A.get(_D):raise CLIError("'AWS_DEFAULT_REGION' must bet set in environment.")
		if not A.get(_B):raise CLIError("'AWS_ACCESS_KEY_ID' must bet set in environment.")
		if not A.get(_C):raise CLIError("'AWS_SECRET_ACCESS_KEY' must bet set in environment.")
	return A
def get_target_config(access_key='',region_name=''):
	C=region_name;B=access_key;A=get_aws_env_config('TARGET')
	if B:A['aws_access_key']=B
	if C:A[_D]=C
	return A
def get_replicator_url():_assert_host_reachable();return f"{config.external_service_url()}/_localstack/replicator"
@click.group(name='replicator',short_help='Start a replication job or check its status',help='\n     The replicator command group allows you to replicate AWS resources into LocalStack.\n     Requires LocalStack to be started with ENABLE_REPLICATOR=1\n    ',cls=RequiresLicenseGroup,hidden=not ENABLE_REPLICATOR)
def replicator():0
@replicator.command(name='start',short_help='Replicate an AWS resource',help='\n    Starts a job to replicate an AWS resource into localstack.\n    You must have credentials with sufficient read access to the resource trying to replicate.\n    At the moment only environment variables are recognized.\n    `AWS_ACCESS_KEY_ID`, `AWS_SECRET_ACCESS_KEY` and `AWS_DEFAULT_REGION` must be set. `AWS_ENDPOINT_URL` and `AWS_SESSION_TOKEN` are optional.\n    ')
@click.option('--replication-type',type=click.Choice(['MOCK','SINGLE_RESOURCE']),help='Type of replication job: MOCK, SINGLE_RESOURCE')
@click.option('--resource-arn',help='ARN of the resource to recreate. Mandatory for SINGLE_RESOURCE replication')
@click.option('--target-account-id',help='Localstack account ID where the resources will be replicated. Defaults to 000000000000. See <docs> to enable same account replication')
@click.option('--target-region-name',help='Localstack region where the resources will be replicated. Only provide if different than source AWS account.')
@click.option(_F,help='Delay for the MOCK replication work')
def start(replication_type,resource_arn=_A,delay=_A,target_account_id=_A,target_region_name=_A):
	E=delay;D=resource_arn;C=replication_type;F=get_source_config();G=get_target_config(access_key=target_account_id,region_name=target_region_name);B={}
	if D:B['resource_arn']=D
	if C=='MOCK':B['delay']=float(E)if E else 1
	H=f"{get_replicator_url()}/jobs";I={'replication_type':C,'replication_job_config':B,'source_aws_config':F,'target_aws_config':G};A=requests.post(H,json=I)
	if A.status_code==200:console.print(A.text)
	else:raise CLIError(f"Failed to create replication job: {A.status_code}, {A.text}")
@replicator.command(name='status',short_help='Check replication status',help='\n    Check the status of a replication job using its Job ID.\n    Use the --follow flag to continuously check the status until the job is completed.\n    ')
@click.argument('job_id')
@click.option('--follow',is_flag=True,help='Follow the status until completed')
@click.option(_F,help='Delay between calls',default=5,type=int)
def status(job_id,follow,delay):
	D=f"{get_replicator_url()}/jobs/{job_id}"
	while True:
		A=requests.get(D)
		if A.status_code==200:
			B=A.json();console.print(B);C=B.get('state')
			if C=='ERROR':raise CLIError(B.get('error_message'))
			elif C=='SUCCEEDED':return
		else:raise CLIError(f"Failed to replicate resource: {A.status_code}, {A.text}")
		if not follow:return
		time.sleep(float(delay))

==> localstack_ext-4.1.2.dev58.tar.gz: localstack_ext-4.1.2.dev58/localstack/pro/core/cli/replicator.py <==
_G='--delay'
_F='SINGLE_RESOURCE'
_E='region'
_D='region_name'
_C='aws_secret_access_key'
_B='aws_access_key_id'
_A=None
import json,os,subprocess as sp,sys,time
from configparser import ConfigParser
from pathlib import Path
from typing import Mapping,Optional,TypedDict
import click,requests
from localstack import config
from localstack.cli import console
from localstack.cli.exceptions import CLIError
from..config import ENABLE_REPLICATOR
from.cli import RequiresLicenseGroup,_assert_host_reachable
AWS_CONFIG_ENV_VARS={_B:'{}_ACCESS_KEY_ID',_C:'{}_SECRET_ACCESS_KEY','aws_session_token':'{}_SESSION_TOKEN',_D:'{}_DEFAULT_REGION','endpoint_url':'{}_ENDPOINT_URL','profile_name':'{}_PROFILE'}
class ProfileLoadError(RuntimeError):
	def __init__(A,profile_name):super().__init__(f"Could not find profile '{profile_name}'")
class AWSConfig(TypedDict,total=False):aws_access_key_id:str;aws_secret_access_key:str;aws_session_token:Optional[str];region_name:str;endpoint_url:Optional[str];profile_name:str
def get_aws_env_config(prefix):A={A:os.getenv(B.format(prefix))for(A,B)in AWS_CONFIG_ENV_VARS.items()};return AWSConfig(**{B:A for(B,A)in A.items()if A})
def get_config_from_profile(profile_name,profile_dir=_A):
	B=profile_name;A=profile_dir;A=A or Path.home()/'.aws'
	def C(path,profile_prefix=''):
		A=ConfigParser();A.read(path)
		try:return A[f"{profile_prefix}{B}"]
		except KeyError:raise ProfileLoadError(B)
	E=C(A/'config',profile_prefix='profile ');D=C(A/'credentials');return AWSConfig(aws_access_key_id=D[_B],aws_secret_access_key=D[_C],region_name=E[_E],profile_name=B)
def get_awscli_config():
	G='configure';F='aws'
	try:A=[F,G,'export-credentials'];B=sp.check_output(A,stderr=sp.PIPE);C=json.loads(B.decode('utf8'))
	except sp.CalledProcessError as D:
		if b'AWS CLI version 2'in D.stderr:print('Warning: awscli v1 installed. Please use v2 for auto detection of credentials',file=sys.stderr);return
	try:
		A=[F,G,'list'];B=sp.check_output(A,stderr=sp.PIPE)
		for E in B.decode().splitlines():
			if _E not in E:continue
			H=E.split()
			try:I=H[1];return AWSConfig(aws_access_key_id=C['AccessKeyId'],aws_secret_access_key=C['SecretAccessKey'],aws_session_token=C.get('SessionToken'),region_name=I)
			except IndexError:return
	except(sp.CalledProcessError,FileNotFoundError)as D:return
def get_source_config(profile_dir=_A):
	B=get_awscli_config()
	if B:print('Configured credentials from the AWS CLI',file=sys.stderr);return B
	A=get_aws_env_config('AWS')
	if not A.get(_D):raise CLIError("'AWS_DEFAULT_REGION' must bet set in environment.")
	if not A.get(_B):raise CLIError("'AWS_ACCESS_KEY_ID' must bet set in environment.")
	if not A.get(_C):raise CLIError("'AWS_SECRET_ACCESS_KEY' must bet set in environment.")
	return A
def get_target_config(access_key='',region_name=''):
	C=region_name;B=access_key;A=get_aws_env_config('TARGET')
	if B:A[_B]=B
	if C:A[_D]=C
	return A
def get_replicator_url():_assert_host_reachable();return f"{config.external_service_url()}/_localstack/replicator"
@click.group(name='replicator',short_help='Start a replication job or check its status',help='\n     The replicator command group allows you to replicate AWS resources into LocalStack.\n     Requires LocalStack to be started with ENABLE_REPLICATOR=1\n    ',cls=RequiresLicenseGroup,hidden=not ENABLE_REPLICATOR)
def replicator():0
@replicator.command(name='start',short_help='Replicate an AWS resource',help='\n    Starts a job to replicate an AWS resource into localstack.\n    You must have credentials with sufficient read access to the resource trying to replicate.\n    At the moment only environment variables are recognized.\n    `AWS_ACCESS_KEY_ID`, `AWS_SECRET_ACCESS_KEY` and `AWS_DEFAULT_REGION` must be set. `AWS_ENDPOINT_URL` and `AWS_SESSION_TOKEN` are optional.\n    ')
@click.option('--replication-type',type=click.Choice(['MOCK',_F]),default=_F,show_default=True,help='Type of replication job: MOCK, SINGLE_RESOURCE')
@click.option('--resource-arn',help='ARN of the resource to recreate. Mandatory for SINGLE_RESOURCE replication')
@click.option('--target-account-id',help='Localstack account ID where the resources will be replicated. Defaults to 000000000000. See <docs> to enable same account replication')
@click.option('--target-region-name',help='Localstack region where the resources will be replicated. Only provide if different than source AWS account.')
@click.option(_G,help='Delay for the MOCK replication work')
def start(replication_type,resource_arn=_A,delay=_A,target_account_id=_A,target_region_name=_A):
	E=delay;D=resource_arn;C=replication_type;F=get_source_config();G=get_target_config(access_key=target_account_id,region_name=target_region_name);B={}
	if D:B['resource_arn']=D
	if C=='MOCK':B['delay']=float(E)if E else 1
	H=f"{get_replicator_url()}/jobs";I={'replication_type':C,'replication_job_config':B,'source_aws_config':F,'target_aws_config':G};A=requests.post(H,json=I)
	if A.status_code==200:console.print(A.text)
	else:raise CLIError(f"Failed to create replication job: {A.status_code}, {A.text}")
@replicator.command(name='status',short_help='Check replication status',help='\n    Check the status of a replication job using its Job ID.\n    Use the --follow flag to continuously check the status until the job is completed.\n    ')
@click.argument('job_id')
@click.option('--follow',is_flag=True,help='Follow the status until completed')
@click.option(_G,help='Delay between calls',default=5,type=int)
def status(job_id,follow,delay):
	D=f"{get_replicator_url()}/jobs/{job_id}"
	while True:
		A=requests.get(D)
		if A.status_code==200:
			B=A.json();console.print(B);C=B.get('state')
			if C=='ERROR':raise CLIError(B.get('error_message'))
			elif C=='SUCCEEDED':return
		else:raise CLIError(f"Failed to replicate resource: {A.status_code}, {A.text}")
		if not follow:return
		time.sleep(float(delay))
